(define-module (popgen pom))
(export pomfun)
(use-modules (srfi srfi-1))

;; global parameters
(define *sp* 0.01)
(define *st* 0.4)
(define *alpha* 4.0)
(define *r* 0.5)
(define *sstar* (- (* *alpha* (- 1 *st*)) *st*))

(define (pomfun freqs)
  ;; this function should define the frequency-independent costly
  ;; female choice model
  ;; 
  ;; selection coefficients
  (let* ((ptm (vector-ref freqs 0))
	 (ppf (vector-ref freqs 1))
	 (Dtmpf (vector-ref freqs 2))
	 (wbarf (- 1 (* *sp* ppf)))
	 (wbarm (- 1 (* *st* ptm)))
	 (wbstar (+ 1 (* *sstar* ptm)))
	 (btp (* *alpha* (- 1 *st*)
		 (/ (- 1 *sp*)
		    (* wbarf wbarm wbstar))))
	 (atmpf btp)
	 (atm (- (* btp ppf)
		 (/ *st*
		    (* wbarf wbarm))))
	 (apf (/ (- (* *sp* ppf)) wbarf))
	 (rr (- 1.0 *r*))
	 (r2d2 (/ rr 2.0))
	 (r2 (/ *r* 2.0)))
    (vector
     ;; Eqn for ptm
     (+ ptm (* 0.5 (+ (* atm (* ptm (- 1 ptm)))
		      (* apf Dtmpf))))
     ;; Eqn for ppf
     (+ ppf (* 0.5 (+ (* apf (* ppf (- 1 ppf)))
		      (* atm Dtmpf))))
     ;; Eqn for Dtmpf
     (+ (* rr Dtmpf)
	(* r2d2 atm (- 1.0 (* 2.0 ptm)) Dtmpf)
	(* r2d2 apf (- 1.0 (* 2.0 ppf)) Dtmpf)
	(* r2 btp (* Dtmpf Dtmpf))
	(* r2 btp ptm (- 1 ptm) ppf (- 1 ppf))))))
