(define-module (popgen popgen))
(export sim-control *every*)

(use-modules (ice-9 format))
(define *numprec*  "~9,8,,,0f ")
(define *numformat* (string-append "~{" *numprec* "~}~%"))
(define *tol* 1e-8)
(define *every* 50)

(define (sim-control popfun inits)
  ;; SIM-CONTROL takes a list of vectors and cdrs down that list by
  ;; evaluating (popgen popfun (car inits))
  (cond ((null? inits) '())
	((not (vector? (car inits)))
	 (scm-error 'wrong-type-arg "sim-control" #f #f (car inits)))
	(else (begin
		(popgen popfun (car inits) *every*)
		(sim-control popfun (cdr inits))))))		    

(define (popgen popfun data every)
  ;; POPFUN is a function that evaluates the vector DATA to yield the
  ;; next generation's data; print the data to the terminal and then
  ;; check the result of evaluating (popfun data); if the new vector
  ;; is close enough to the old one, exit and return #f; otherwise
  ;; call (popgen POPFUN (POPFUN DATA))
  ;;
  ;; First print the data to the current-output-port (user routines
  ;; may want to modify this to send the data down a pipe)
  (let* ((new (popfun data))
	 (diff (eudiff new data)))
    (cond ((< diff *tol*) (format #t "~%~%"))
	  ((= every 0)
	   (format #t *numformat* (vector->list data))
	   (popgen popfun new every))
	  (else	  
	   (popgen popfun new (1- every))))))

(define (eudiff vec1 vec2)
  (sqrt (apply
	 +
	 (map (lambda (x y)
		(* (- x y) (- x y)))
	      (vector->list vec1)
	      (vector->list vec2)))))